package jingdong

import (
	"context"
	"github.com/gogf/gf/encoding/gjson"
	"github.com/gogf/gf/frame/g"
)

type orderJD struct {
}

//Order 订单
var Order = orderJD{}

//GetDeliveryRes 物流轨迹
type GetDeliveryRes struct {
	*CommonRes
	Result struct {
		OrderTrack []struct {
			Content  string `json:"content"`
			MsgTime  string `json:"msgTime"`
			Operator string `json:"operator"`
		} `json:"orderTrack"`
		WaybillCode []struct {
			OrderID         string `json:"orderId"`
			ParentID        string `json:"parentId"`
			Carrier         string `json:"carrier"`
			DeliveryOrderID string `json:"deliveryOrderId"`
		} `json:"waybillCode"`
		JdOrderID int `json:"jdOrderId"`
	} `json:"result"`
}

//PriceSnap 商品价格
type PriceSnap struct {
	SkuID int `json:"skuId"`
	Price int `json:"price"`
}

//PlaceOrderReq 下单
type PlaceOrderReq struct {
	OrderSn   string
	SkuNums   []*SkuNums
	Consignee string
	Phone     string
	Address   *PlaceOrderAddress
	PriceSnap []*PriceSnap
}

//PlaceOrderAddress 下单地址
type PlaceOrderAddress struct {
	Province string
	City     string
	County   string
	Town     string
	Desc     string
}

//PlaceOrderRes 下单
type PlaceOrderRes struct {
	*CommonRes
	Result struct {
		JdOrderID       int64           `json:"jdOrderId"`
		OrderPrice      float64         `json:"orderPrice"`
		OrderNakedPrice float64         `json:"orderNakedPrice"`
		OrderTaxPrice   float64         `json:"orderTaxPrice"`
		Sku             []*OrderSkuItem `json:"sku"`
	} `json:"result"`
}

//OrderSkuItem 订单商品
type OrderSkuItem struct {
	SkuID      int     `json:"skuId"`
	Num        int     `json:"num"`
	Price      float64 `json:"price"`
	Name       string  `json:"name"`
	NakedPrice float64 `json:"nakedPrice"`
}

//ReflectOrderRes 反查订单
type ReflectOrderRes struct {
	*CommonRes
	Result string `json:"result"`
}

//GetOrderRes 订单详情
type GetOrderRes struct {
	*CommonRes
	Result struct {
		JdOrderID       int64           `json:"jdOrderId"`
		Type            int             `json:"type"`
		Freight         float64         `json:"freight"`
		OrderPrice      float64         `json:"orderPrice"`
		OrderNakedPrice float64         `json:"orderNakedPrice"`
		OrderTaxPrice   float64         `json:"orderTaxPrice"`
		Sku             []*OrderSkuItem `json:"sku"`
		OrderState      int             `json:"orderState"`
		State           int             `json:"state"`
		POrder          struct {
			JdOrderID       int64           `json:"jdOrderId"`
			Freight         float64         `json:"freight"`
			OrderPrice      float64         `json:"orderPrice"`
			OrderNakedPrice float64         `json:"orderNakedPrice"`
			Sku             []*OrderSkuItem `json:"sku"`
		} `json:"pOrder"`
		COrder []struct {
			POrder          string         `json:"pOrder"`
			JdOrderID       int            `json:"jdOrderId"`
			Freight         float64        `json:"freight"`
			OrderPrice      float64        `json:"orderPrice"`
			OrderNakedPrice float64        `json:"orderNakedPrice"`
			Sku             []OrderSkuItem `json:"sku"`
		} `json:"cOrder"`
	} `json:"result"`
}

//GetFreightRes 运费
type GetFreightRes struct {
	*CommonRes
	Result struct {
		Freight             float64 `json:"freight"`
		BaseFreight         float64 `json:"baseFreight"`
		RemoteRegionFreight float64 `json:"remoteRegionFreight"`
	} `json:"result"`
}

//SkuNums 商品数量
type SkuNums struct {
	SkuID int `json:"skuId"`
	Num   int `json:"num"`
}

// GetDelivery 物流信息
func (*orderJD) GetDelivery(ctx context.Context, OrderID string) (res *GetDeliveryRes, err error) {
	method := "order/orderTrack"
	param := g.MapStrStr{
		"jdOrderId":   OrderID,
		"waybillCode": "1",
	}
	result, err := server.requestApi(ctx, method, param)
	if err != nil {
		return
	}
	err = gjson.New(result).Scan(&res)
	return
}

// PlaceOrder 下单
func (orderJD) PlaceOrder(ctx context.Context, req *PlaceOrderReq) (res *PlaceOrderRes, err error) {
	method := "order/submitOrder"
	var param = g.MapStrStr{
		"thirdOrder":           req.OrderSn,
		"sku":                  gjson.New(req.SkuNums).MustToJsonString(),
		"orderPriceSnap":       gjson.New(req.PriceSnap).MustToJsonString(),
		"name":                 req.Consignee,
		"mobile":               req.Phone,
		"province":             req.Address.Province,
		"city":                 req.Address.City,
		"county":               req.Address.County,
		"town":                 req.Address.Town,
		"address":              req.Address.Desc,
		"email":                server.Email,
		"companyName":          server.CompanyName,
		"submitState":          server.SubmitState,
		"invoicePhone":         server.Mobile,
		"invoiceState":         "2",
		"invoiceType":          "4",
		"selectedInvoiceTitle": "5",
		"invoiceContent":       "1",
		"paymentType":          "4",
		"isUseBalance":         "1",
		"doOrderPriceMode":     "1",
	}
	result, err := server.requestApi(ctx, method, param)
	if err != nil {
		return
	}
	err = gjson.New(result).Scan(&res)
	return
}

// ReflectOrder 反查订单
func (orderJD) ReflectOrder(ctx context.Context, thirdOrder string) (res *ReflectOrderRes, err error) {
	method := "order/selectJdOrderIdByThirdOrder"
	param := g.MapStrStr{
		"thirdOrder": thirdOrder,
	}
	result, err := server.requestApi(ctx, method, param)
	if err != nil {
		return
	}
	err = gjson.New(result).Scan(&res)
	return
}

// GetOrderInfo 订单详情
func (orderJD) GetOrderInfo(ctx context.Context, OrderID string) (res *GetOrderRes, err error) {
	method := "order/selectJdOrder"
	param := g.MapStrStr{
		"jdOrderId": OrderID,
	}
	result, err := server.requestApi(ctx, method, param)
	if err != nil {
		return
	}
	_ = gjson.New(result).Scan(&res)
	return
}

// GetFreight 查询运费
func (orderJD) GetFreight(ctx context.Context, sku []*SkuNums, province, city, county, town string) (res *GetFreightRes, err error) {
	method := "order/getFreight"
	param := g.MapStrStr{
		"sku":         gjson.New(sku).MustToJsonString(),
		"province":    province,
		"city":        city,
		"county":      county,
		"town":        town,
		"paymentType": "4",
	}
	result, err := server.requestApi(ctx, method, param)
	if err != nil {
		return
	}
	err = gjson.New(result).Scan(&res)
	return
}

// ConfirmReceived 确认收货
func (orderJD) ConfirmReceived(ctx context.Context, OrderID string) (res *CommonRes, err error) {
	method := "order/confirmReceived"
	param := g.MapStrStr{
		"jdOrderId": OrderID,
	}
	result, err := server.requestApi(ctx, method, param)
	if err != nil {
		return
	}
	err = gjson.New(result).Scan(&res)
	return

}
